
from .precomp import precomp
from .precomp import params_kagra
from .plot    import plot_noise

from . import noise as noise

from collections import OrderedDict
import copy

from struct import Struct

def load_ifo(path):
    return Struct.from_file(path)

def noise_calc(freq, ifo, option = None):
    """Calculate all IFO noises and return as dict

    Assumes ifo has already been run through precompIFO().

    """
    noises = OrderedDict()

    keys = [ 'Quantum Vacuum', 'Suspension Thermal', 'Coating Brownian',
            #'Coating Thermo-Optic'
             'Substrate Thermo-Elastic', 'Substrate Brownian', 'Seismic',
             'Newtonian Gravity',
            #'Excess Gas'
             'Excess Noise' ] # Excess noise added

    funcs = [ noise.quantum.shotrad, noise.suspensionthermal.susptherm, 
              noise.coatingthermal.coatbrownian, 
             #noise.coatingthermal.thermooptic,
              noise.substratethermal.subtherm,
              noise.substratethermal.subbrownian, noise.seismic.seismic,
              noise.newtonian.gravg, 
             #noise.residualgas.gas,
              noise.technical.excess ]

    if option is not None and option.lower().find('qonly') >= 0:
        keys  = [ 'Quantum Vacuum' ]
        funcs = [ noise.quantum.shotrad ]

    if ifo.get('Technical', 0) == 0:
        keys  = keys [:-1]
        funcs = funcs[:-1]

    for k,f in zip(keys, funcs):
        noises[k] = f(freq, ifo)

    noises['Total'] = sum(noises[curve] for curve in noises)
    noises['Freq'] = freq

    return noises

def gwinc(freq, ifoin, conf = None, verbose = False, option = None):
    """Calculate strain noise budget for a specified interferometer model.

    Argument `freq` is the frequency array for which the noises will
    be calculated, and `ifoin` is the IFO model (see the `load_ifo()`
    function).

    Returns tuple of (noises, ifo)

    """
    if conf is not None:
        ifo = params_kagra(ifoin, conf)
    else:
        ifo = copy.deepcopy(ifoin)

    title = 'None'
    if verbose:
        if conf is not None:
            title = 'bKAGRA ('+conf+')' if conf.find('RSE') >= 0\
                                        or conf.find('O3')  >= 0\
               else 'KAGRA+ ('+conf+')'
        else:
            title = verbose

    # add some precomputed info to the ifo struct
    #this implicitly deepcopies and the return value is the copy
    ifo = precomp(freq, ifo, title, option)

    noises = noise_calc(freq, ifo, option)

    if option is not None and option.lower().find('ifo') >= 0:
        return noises, ifo

    return noises
