
import matplotlib.pyplot as plt

def plot(f, p, fig=None,
         fsiz=18, xmin=10, xmax=5000, ymin=1e-25, ymax=1e-22,
         loc='upper left', ncol=1, bba=0, **args):
    if fig is None:
        fig = plt.figure()

    plt.subplots_adjust(top=0.90, bottom=0.15, left=0.15, right=0.95)

    ax = fig.add_subplot(1, 1, 1)

    if xmin is not None and xmax is not None:
        ax.set_xlim([xmin, xmax])

    if ymin is not None and ymax is not None:
        ax.set_ylim([ymin, ymax])

    ax.xaxis.grid(True)
    ax.yaxis.grid(True)

    ax.loglog(f, p, **args)

    plt.tick_params(labelsize = fsiz)

    if bba > 0:
        ax.legend(fontsize=fsiz, loc=loc, ncol=ncol, bbox_to_anchor=(1, bba))
    else:
        ax.legend(fontsize=fsiz, loc=loc, ncol=ncol)

    ax.set_xlabel('Frequency (Hz)',         fontsize=fsiz)
    ax.set_ylabel('Strain (1/$\sqrt{Hz}$)', fontsize=fsiz)

    return fig

