
import lal
import numpy as np
from scipy import interpolate

def SimInspiralGetApproximantFromString(apprx):
    return 'IMRPhenomD' if apprx == 'IMRPhenomD' else 'TaylorF2'

class Data:
    pass

class Model:
    pass

def SimInspiralChooseFDWaveform(**params):
    apprx = params['approximant']
    d     = params['distance']/lal.PC_SI
    m1    = params['m1']/lal.MSUN_SI
    m2    = params['m2']/lal.MSUN_SI
    m     = (m1+m2)/2

    df = params['deltaF']
    f0 = params['f_min']
    fm = params['f_max']

    if f0 < 0.5:
        f0 = 0.5

    h = IMR     (m=m, d=d, df=df, f0=f0) if apprx == 'IMRPhenomD' else \
        TaylorF2(m=m, d=d, df=df, f0=f0)

    n = int((fm-f0)/df)
    h = np.append(h, np.zeros(n-len(h)))

    m = Model()
    m.f0 = f0
    m.deltaF = df
    m.data = Data()
    m.data.data = h

    return m,m

def TaylorF2(m=1.4, d=100e6, df=0.5, f0=1):
    Mtsun = 4.925491025543575903411922162094833998e-6
    Mrsun = 1.476625061404649406193430731479084713e3
    Pc    = 3.085677581491367278913937957796471611e16
    Pi    = np.pi

    visco = 1/np.sqrt(6)
    fisco = visco*visco*visco/(m+m)/Mtsun/Pi

    d    *= Pc
    eta   = 1./4
    ftan  = 32*eta*eta/5
    detan = -eta
    amp0  = 4*m*m/d*Mrsun*Mtsun*np.sqrt(Pi/12)
    pim   = Pi*m*2*Mtsun

    n   = np.floor(fisco/df)
    x   = np.linspace(f0, n*df, n, endpoint=True)

    v   = np.cbrt(pim*x)
    v10 = np.power(v, 10)
    flx = ftan*v10
    de  = detan*v
    y   = amp0*np.sqrt(-de/flx)*v

    i   = np.where(10 <= x)[0][0]

    return y

def IMR(m=30, d=100e6, df=0.5, f0=1):
    """
    Parameterization of IMRPhenomD/P waveform amplitude
    assuming no spin (a1=a2=0) and same mass (m1=m2=m)
    """

    # Total mass
    mt = m*2
    # Ms/c^3
    ms = 4.925491025543575903411922162094833998e-6
    # Spline nodes
    xp = [ -10,    -9,   -8,    -7,   -6,    -5,   -4.7, -4.5, -4.4, -4.27, \
            -4.27, -4,   -3.7,  -3.5, -3.2,  -3,   -2.8, -2.7, -2.6, -2.554,\
            -2.55, -2.5, -2.45, -2.4, -2.35, -2.3, -2.2, -2.1, -2,   -1.9,  \
            -1.8, -1.7, -1.6 ]
    yp = [  6.795,   5.731,   4.539,  3.364,  2.177,\
            0.955,   0.572,   0.310,  0.177,  0.002,\
            0.002,  -0.360,  -0.743, -0.971, -1.243,\
           -1.372,  -1.477,  -1.536, -1.615, -1.663,\
           -1.667,  -1.751,  -1.913, -2.193, -2.602,\
           -3.118,  -4.312,  -5.564, -6.822, -8.086,\
           -9.375, -10.705, -12.091 ]
    # Amplitude scaling factor from compnent mass 
    mm = 2*(np.log(m)-3.4)-50.5818

    n  = np.floor((np.exp(xp[-1])/mt/ms)/df)
    x  = np.linspace(f0, n*df, n, endpoint=True)
    lx = np.log(x*mt*ms)

    i1 = np.where(lx < xp[ 9])[0][-1]+1
    i2 = np.where(lx < xp[19])[0][-1]+1
    i3 = np.where(lx < xp[20])[0][-1]+1
    i4 = np.where(lx < xp[-1])[0][-1]+1

    f1 = interpolate.interp1d(xp[ 0:10], yp[ 0:10], kind="cubic")
    f2 = interpolate.interp1d(xp[10:20], yp[10:20], kind="cubic")
    f3 = interpolate.interp1d(xp[19:21], yp[19:21], kind="linear")
    f4 = interpolate.interp1d(xp[20:33], yp[20:33], kind="cubic")

    y = f1(lx[0:i1])
    y = np.append(y, f2(lx[i1:i2]))
    y = np.append(y, f3(lx[i2:i3]))
    y = np.append(y, f4(lx[i3:i4]))

    y = np.exp(y+mm) *100e6/d

    return y
